var Functions = require('../../Functions');
var SubMenu = require('../../gui/SubMenu');


function MolStar(containerParentElement, customMap) {

  var molstarDiv = Functions.createElement({
    type: "div",
    id: "minervaMolStarContainer",
    className: "minerva-molstar-container"
  });
  containerParentElement.appendChild(molstarDiv);

  this.setContainerElement(molstarDiv);
  this._customMap = customMap;
}

MolStar.prototype.setContainerElement = function (containerElement) {
  this._containerElement = containerElement;
};

MolStar.prototype.getContainerElement = function () {
  return this._containerElement;
};

function removeFromContextMenu(menu) {
  $(menu.getElement()).find('li:contains("MolStar")').remove();
}

MolStar.prototype.activateInContextMenu = function (uniprotIds) {

  var self = this;
  var menu = this._customMap.getContextMenu();
  removeFromContextMenu(menu);

  var submenu = new SubMenu({
    element: Functions.createElement({type: "li"}),
    name: "Open MolStar",
    customMap: self._customMap
  });
  uniprotIds.forEach(function (uniprotId) {
    submenu.addOption(uniprotId, function () {
      self._activate(uniprotId);
    }, false);
  });
  menu.addOption(submenu);

};

MolStar.prototype.deactivateInContextMenu = function () {
  var menu = this._customMap.getContextMenu();
  removeFromContextMenu(menu);
  menu.addOption("Open MolStar (no UniProt ID available)", function () {
  }, true);
};

MolStar.prototype._deactivate = function () {
  var container = this.getContainerElement();
  container.style.display = 'none';
  $(container).empty();
  this.molStar.destroy();
  this.molStar = undefined;
};

MolStar.prototype._activate = function (uniprotId) {

  var self = this;

  var container = this.getContainerElement();
  if (!container) return;

  var maxZIndex = Math.max.apply(null,
    $.map($('body *'), function (e) {
      if ($(e).css('position') !== 'static') {
        return parseInt($(e).css('z-index')) || 1;
      }
    }));

  $(container).css('z-index', maxZIndex + 1);

  var molStarCloseButton = Functions.createElement({
    type: "div",
    className: "minerva-molstar-close-button",
    content: 'x'
  });
  molStarCloseButton.addEventListener('click', function () {
    self._deactivate();
    return false;
  });
  container.appendChild(molStarCloseButton);

  var MolStarPlugin = require('molstar');

  this.molStar = new MolStarPlugin({
    uniprotId: uniprotId,
    containerId: container.id
  });
  container.style.display = 'block';
};


module.exports = MolStar;
